# Copyright 2024 Q-CTRL. All rights reserved.
#
# Licensed under the Q-CTRL Terms of service (the "License"). Unauthorized
# copying or use of this file, via any medium, is strictly prohibited.
# Proprietary and confidential. You may not use this file except in compliance
# with the License. You may obtain a copy of the License at
#
#    https://q-ctrl.com/terms
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS. See the
# License for the specific language.

from qctrlworkflowclient.utils import PackageInfo as _PackageInfo

INVALID_SUBSCRIPTION_ERROR = """
---------------------------------------------------------------------
Your authentication succeeded, but your subscription is invalid for
this product.

Please access `https://fire.q-ctrl.com` for more details.
---------------------------------------------------------------------
"""

FIRE_OPAL_USER_ROLE = "fire-opal-cli-access"

API_KEY_NAME = "QCTRL_API_KEY"

PACKAGE_INFO = _PackageInfo(
    name="Fire Opal client",
    install_name="fire-opal",
    import_name="fireopal",
    changelog_url="https://docs.q-ctrl.com/fire-opal/changelog",
)
